// This sample demonstrates handling intents from an Alexa skill using the Alexa Skills Kit SDK (v2).
// Please visit https://alexa.design/cookbook for additional examples on implementing slots, dialog management,
// session persistence, api calls, and more.
const Alexa = require('ask-sdk-core');
const https = require('https'); 
//launch gmail skill
const LaunchRequestHandler = {
canHandle(handlerInput) {
return handlerInput.requestEnvelope.request.type === 'LaunchRequest';
},
handle(handlerInput) {
//outputs to the user
const speechText = 'Welcome to GMail, you can say things like Do I have any new mail and How many emails have I received today';
return handlerInput.responseBuilder
.speak(speechText)
.reprompt(speechText)
.getResponse();
}
};
// calls MailCountIntent
const MailCountIntentHandler ={ 
canHandle(handlerInput){
return handlerInput.requestEnvelope.request.type === 'IntentRequest'
&& handlerInput.requestEnvelope.request.intent.name === 'MailCountIntent';
},
handle(handlerInput){
return new Promise((resolve)=>{
getMailCount(handlerInput, (theResult)=>{
const sessionAttributes = handlerInput.attributesManager.getSessionAttributes();
// checks users logged into google
console.log("Logging " + theResult);
// gives the user the output
const output = `You have ${theResult} new email(s), would you like me to read this back to you?`;
handlerInput.attributesManager.setSessionAttributes(sessionAttributes);
//outputs title for samle card as well as the result of the users request
resolve(handlerInput.responseBuilder.speak(output).reprompt(output).withSimpleCard('GMail', output).getResponse());
});
});
}
};
const MailTotalIntentHandler ={ 
canHandle(handlerInput){
return handlerInput.requestEnvelope.request.type === 'IntentRequest'
&& handlerInput.requestEnvelope.request.intent.name === 'MailTotalIntent';
},
handle(handlerInput){
return new Promise((resolve)=>{
getMailTotal(handlerInput, (theResult)=>{
console.log("Logging " + theResult);
const output = `You have ${theResult} email(s) in total`;
resolve(handlerInput.responseBuilder.speak(output).withSimpleCard('Gmail', output).getResponse());
});
});
}
};
// calls MailCountTimeIntent
const MailCountTimeIntentHandler ={ 
canHandle(handlerInput){
return handlerInput.requestEnvelope.request.type === 'IntentRequest'
&& handlerInput.requestEnvelope.request.intent.name === 'MailCountTimeIntent';
},
handle(handlerInput){
return new Promise((resolve)=>{
//undefined "getMailCountTime"
getMailCountTime(handlerInput, (theResult)=>{
// checks users logged into google
console.log("Logging " + theResult);
// gives the user the output
const output = `You have ${theResult.resultSizeEstimate} new email(s) today, would you like me to read for you?`;
const sessionAttributes = handlerInput.attributesManager.getSessionAttributes();
sessionAttributes.messages = theResult.messages ; 
sessionAttributes.index = 0 ;
handlerInput.attributesManager.setSessionAttributes(sessionAttributes); // just noticed that you need to write this here I think.
//outputs title for samle card as well as the result of the users request
resolve(handlerInput.responseBuilder.speak(output).withSimpleCard('GMail', output).getResponse());
});
});
}
};
// calls YesIntent
const YesIntentHandler ={ 
canHandle(handlerInput){
return handlerInput.requestEnvelope.request.type === 'IntentRequest'
&& handlerInput.requestEnvelope.request.intent.name === 'YesIntent';
},
handle(handlerInput) {
const sessionAttributes = handlerInput.attributesManager.getSessionAttributes();
const { messages } = sessionAttributes;
let speechText = '';
if (typeof messages === 'undefined') {
speechText = 'no messages yet';
} else {
return new Promise((resolve)=>{
getFullMessage(handlerInput, (theResult)=>{
// checks users logged into google
console.log("Logging " + theResult);
// gives the user the output
const output = `${theResult}`;
//outputs title for samle card as well as the result of the users request
resolve(handlerInput.responseBuilder.speak(output).withSimpleCard('GMail', output).getResponse());

});
});
}
}
};
// calls NoIntent
const NoIntentHandler ={ 
canHandle(handlerInput){
return handlerInput.requestEnvelope.request.type === 'IntentRequest'
&& handlerInput.requestEnvelope.request.intent.name === 'NoIntent';
},
handle(handlerInput) {
//outputs to the user
const speechText = 'Okay';
return handlerInput.responseBuilder
.speak(speechText)
.reprompt(speechText)
.getResponse();
}
};
//calls NextIntent
const NextIntentHandler = {
  canHandle(handlerInput){
return handlerInput.requestEnvelope.request.type === 'IntentRequest'
&& handlerInput.requestEnvelope.request.intent.name === 'NextIntent';
},
handle(handlerInput) {
const sessionAttributes = handlerInput.attributesManager.getSessionAttributes();
const { messages } = sessionAttributes;
let speechText = '';
if (typeof messages === 'undefined') {
speechText = 'no messages yet';
} else {
    if(sessionAttributes.index < sessionAttributes.messages.length-1 )
{
sessionAttributes.index=sessionAttributes.index+1;

}

return new Promise((resolve)=>{
getFullMessage(handlerInput, (theResult)=>{
// checks users logged into google
console.log("Logging " + theResult);
// gives the user the output
const output = `${theResult}`;
//outputs title for samle card as well as the result of the users request

resolve(handlerInput.responseBuilder.speak(output).withSimpleCard('GMail', output).getResponse());

});
});
}

}
};
//calls PreviousIntent
const PreviousIntentHandler = {
  canHandle(handlerInput){
return handlerInput.requestEnvelope.request.type === 'IntentRequest'
&& handlerInput.requestEnvelope.request.intent.name === 'PreviousIntent';
},
handle(handlerInput) {
const sessionAttributes = handlerInput.attributesManager.getSessionAttributes();
const { messages } = sessionAttributes;
let speechText = '';
if (typeof messages === 'undefined') {
speechText = 'no messages yet';
} else {
    if(sessionAttributes.index > 0 )
{
sessionAttributes.index=sessionAttributes.index-1;

}
else
speechText='This is the first email!';
return new Promise((resolve)=>{
getFullMessage(handlerInput, (theResult)=>{
// checks users logged into google
console.log("Logging " + theResult);
// gives the user the output
const output = `${theResult}`;
//outputs title for samle card as well as the result of the users request
 
resolve(handlerInput.responseBuilder.speak(output).withSimpleCard('GMail', output).getResponse());


});
});
}

}
};
// calls MessageIntent
const MessageIntentHandler ={ 
canHandle(handlerInput){
return handlerInput.requestEnvelope.request.type === 'IntentRequest'
&& handlerInput.requestEnvelope.request.intent.name === 'MessageIntent';
},
handle(handlerInput){
return new Promise((resolve)=>{
getMessage(handlerInput, (theResult)=>{
// checks users logged into google
console.log("Logging " + theResult);
// gives the user the output
const output = `From:Razer Recived on Thu, 11 Apr: Message: ${theResult}`;
//outputs title for samle card as well as the result of the users request
resolve(handlerInput.responseBuilder.speak(output).withSimpleCard('GMail', output).getResponse());
});
});
}
};
// calls FromIntent
const FromIntentHandler ={ 
canHandle(handlerInput){
return handlerInput.requestEnvelope.request.type === 'IntentRequest'
&& handlerInput.requestEnvelope.request.intent.name === 'FromIntent';
},
handle(handlerInput){
return new Promise((resolve)=>{
//undefined "getMailCountTime"
getMessage(handlerInput, (theResult)=>{
// checks users logged into google
console.log("Logging " + theResult);
// gives the user the output
const output = `You have ${theResult} From Razer`;
//outputs title for samle card as well as the result of the users request
resolve(handlerInput.responseBuilder.speak(output).withSimpleCard('GMail', output).getResponse());
});
});
}
};
function getMailCount(handlerInput,callback){
console.log("mil");
// locates the accessToken from googles api and checks the users inbox
var accessToken = handlerInput.requestEnvelope.context.System.user.accessToken;
let options = {
host: 'www.googleapis.com',
path: "/gmail/v1/users/me/labels/INBOX?access_token="+accessToken
};
var req = https.request(options, res=> {
console.log("get mail");
var responseString = "";
res.on('data', data => {
console.log("data");
responseString += data;
});
res.on('end',()=>{
console.log("end");
console.log(JSON.parse(responseString));
//return JSON.parse(responseString).messagesUnread;
callback(JSON.parse(responseString).messagesUnread);//JSON.parse(responseString).messagesUnread; 
});
}).on('error', (error) => {
console.error(error)
callback(error.message)
//return error.message;
});
req.end();
}
//get mail total
function getMailTotal(handlerInput,callback)
{
console.log("mil");
var accessToken = handlerInput.requestEnvelope.context.System.user.accessToken;
let options = {
host: 'www.googleapis.com',
path: "/gmail/v1/users/me/labels/INBOX?access_token="+accessToken
};
var req = https.request(options, res=> {
console.log("get mail");
var responseString = "";
res.on('data', data => {
console.log("data");
responseString += data;
});
res.on('end',()=>{
console.log("end");
console.log(JSON.parse(responseString));
//return JSON.parse(responseString).messagesUnread;
callback(JSON.parse(responseString).messagesTotal);//JSON.parse(responseString).messagesUnread; 
});
}).on('error', (error) => {
console.error(error)
callback(error.message)
//return error.message;
});
req.end();
}
//getMailCountTime
function getMailCountTime(handlerInput,callback){
const sessionAttributes = handlerInput.attributesManager.getSessionAttributes();
//const { messages } = sessionAttributes;
 
     console.log("mil");
    var accessToken = handlerInput.requestEnvelope.context.System.user.accessToken;
    let options = {
         host: 'www.googleapis.com',
          path: "/gmail/v1/users/me/messages?q=newer_than:2d&access_token="+accessToken
     };
     var req = https.request(options, res=> {
          console.log("get mail");
          var responseString = "";
          res.on('data', data => {
               console.log("data");
              responseString += data;
          });
          res.on('end',()=>{
               console.log("end");
                console.log(JSON.parse(responseString));
                //return JSON.parse(responseString).messagesUnread;
             callback(JSON.parse(responseString));//JSON.parse(responseString).resultSizeEstimate; 
          });
      }).on('error', (error) => {
 
console.error(error)
callback(error.message)
//return error.message;
});
req.end();
}

function getFrom(handlerInput,callback){
const sessionAttributes = handlerInput.attributesManager.getSessionAttributes();
const { messages } = sessionAttributes;
const { index } = sessionAttributes;
console.log("mil");
var accessToken = handlerInput.requestEnvelope.context.System.user.accessToken;
let options = {
host: 'www.googleapis.com',
path: "/gmail/v1/users/me/messages/"+messages[index].id+"?access_token="+accessToken
 
     };
     
     var req = https.request(options, res=> {
          console.log("get mail");
          var responseString = "";
          res.on('data', data => {
               console.log("data");
              responseString += data;
          });
          res.on('end',()=>{
               console.log("end");
                console.log(JSON.parse(responseString));
                
                
                //return JSON.parse(responseString).snippet;
               // const fromString = JSON.parse(responseString).payload.headers[18].value;
              // var mail = JSON.parse(responseString);
             //var headers = mail.payload.headers;
             
                //var subjectString = headers.find(x=> x.name==='Subject').value;JSON.parse(responseString).payload.headers.find(x=> x.name==='From:');//.payload.headers[22].value;
                const fromString = makereadable( JSON.parse(responseString).payload.headers.find(x=> x.name==='From').value);// JSON.parse(responseString).payload.headers[14].value;
                const firstName =fromString.split(" ")[0];
                var secondName="";
                
                 if(fromString.split(" ")[1].split("@").length===1);
                 {
                     secondName=fromString.split(" ")[1];
                 }
                 
                const fullString=  'From:'+ firstName+' '+secondName;
             callback(fullString);//JSON.parse(responseString).snippet;
          });
      }).on('error', (error) => {
 
console.error(error)
callback(error.message)
//return error.message;
});
req.end();
}



function getSubject(handlerInput,callback){
const sessionAttributes = handlerInput.attributesManager.getSessionAttributes();
const { messages } = sessionAttributes;
const { index } = sessionAttributes;
console.log("mil");
var accessToken = handlerInput.requestEnvelope.context.System.user.accessToken;
let options = {
host: 'www.googleapis.com',
path: "/gmail/v1/users/me/messages/"+messages[index].id+"?access_token="+accessToken
 
     };
     
     var req = https.request(options, res=> {
          console.log("get mail");
          var responseString = "";
          res.on('data', data => {
               console.log("data");
              responseString += data;
          });
          res.on('end',()=>{
               console.log("end");
                console.log(JSON.parse(responseString));
                //return JSON.parse(responseString).snippet;
               // const fromString = JSON.parse(responseString).payload.headers[18].value;
               var mail = JSON.parse(responseString);
             //var headers = mail.payload.headers;
                //var subjectString = headers.find(x=> x.name==='Subject').value;JSON.parse(responseString).payload.headers.find(x=> x.name==='From:');//.payload.headers[22].value;
                const subjectString = JSON.parse(responseString).payload.headers.find(x=> x.name==='Subject').value;
                const fullString=  'Subject:'+ subjectString;
             callback(fullString);//JSON.parse(responseString).snippet;
          });
      }).on('error', (error) => {
 
console.error(error)
callback(error.message)
//return error.message;
});
req.end();
}

function getMessage(handlerInput,callback){
const sessionAttributes = handlerInput.attributesManager.getSessionAttributes();
const { messages } = sessionAttributes;
const { index } = sessionAttributes;
console.log("mil");
var accessToken = handlerInput.requestEnvelope.context.System.user.accessToken;
let options = {
host: 'www.googleapis.com',
path: "/gmail/v1/users/me/messages/"+messages[index].id+"?access_token="+accessToken
 
     };
     
     var req = https.request(options, res=> {
          console.log("get mail");
          var responseString = "";
          res.on('data', data => {
               console.log("data");
              responseString += data;
          });
          res.on('end',()=>{
               console.log("end");
                console.log(JSON.parse(responseString));
                //return JSON.parse(responseString).snippet;
               // const fromString = JSON.parse(responseString).payload.headers[18].value;
               var mail = JSON.parse(responseString);
             //var headers = mail.payload.headers;
                //var subjectString = headers.find(x=> x.name==='Subject').value;JSON.parse(responseString).payload.headers.find(x=> x.name==='From:');//.payload.headers[22].value;
                const snippetString = JSON.parse(responseString).snippet;
                const fullString= 'Message:'+ snippetString;
             callback(fullString);//JSON.parse(responseString).snippet;
          });
      }).on('error', (error) => {
 
console.error(error)
callback(error.message)
//return error.message;
});
req.end();
}

function getFullMessage(handlerInput,callback){
const sessionAttributes = handlerInput.attributesManager.getSessionAttributes();
const { messages } = sessionAttributes;
const { index } = sessionAttributes;
console.log("mil");
var accessToken = handlerInput.requestEnvelope.context.System.user.accessToken;
let options = {
host: 'www.googleapis.com',
path: "/gmail/v1/users/me/messages/"+messages[index].id+"?access_token="+accessToken
 
     };
     
     var req = https.request(options, res=> {
          console.log("get mail");
          var responseString = "";
          res.on('data', data => {
               console.log("data");
              responseString += data;
          });
          res.on('end',()=>{
               console.log("end");
                console.log(JSON.parse(responseString));
                const fromString = makereadable( JSON.parse(responseString).payload.headers.find(x=> x.name==='From').value);// JSON.parse(responseString).payload.headers[14].value;
                var fromSplitSrings = fromString.split(" ");
                const firstName =fromSplitSrings[0];
                var secondName=fromSplitSrings[1];
                  const subjectString = makereadable(JSON.parse(responseString).payload.headers.find(x=> x.name==='Subject').value);
               var fullString = 'returned text';
               if(sessionAttributes.index< sessionAttributes.messages.length)
                fullString=  ('From '+ firstName+' '+secondName + ' Subject '+subjectString)+(' Message reads '+JSON.parse(responseString).snippet) +(' Email '+(sessionAttributes.index+1)+' of '+sessionAttributes.messages.length)+', Next to move forward or Previous to go back?';
                else
                 fullString=  'From '+ firstName+' '+secondName + ' Subject '+subjectString+' Message reads '+JSON.parse(responseString).snippet +' Email '+(sessionAttributes.index+1)+' of '+sessionAttributes.messages.length+ '. Last of todays email list, say Previous to go back and read earlier emails.';
                
              //  const fullString= 'From '+ firstName+' '+secondName + ' Message reads '+JSON.parse(responseString).snippet +' would you like me to read the next email';
                //var subjectString = headers.find(x=> x.name==='Subject').value;JSON.parse(responseString).payload.headers.find(x=> x.name==='From:');//.payload.headers[22].value;
               // const subjectString = getSubject(responseString);
               // const fromString = getFrom(responseString);
               // const snippetString = getMessage(responseString);
           //   const fullString= JSON.parse(responseString).snippet;//  fromString+ subjectString + snippetString;
             callback(fullString);//JSON.parse(responseString).snippet;
          });
      }).on('error', (error) => {
 
console.error(error)
callback(error.message)
//return error.message;
});
req.end();
}
 
   //Helper to make strings readable by Alexa
function makereadable(string){
    if(string.length>=7500){string=string.substr(0, 7499);}
    string = string.replace(/&amp/g,"&");
    string = string.replace(/ fwd:/," forward:");
    string = string.replace(/ re:/," reply:");
    string = string.replace(/&/g," and ");
    string = string.replace(/[^a-zA-Z0-9-*.,: @$]/g,"");
    string = string.replace(/\r?\n|\r/g, " ");
    return string;
 }

const HelpIntentHandler = {
canHandle(handlerInput) {
return handlerInput.requestEnvelope.request.type === 'IntentRequest'
&& handlerInput.requestEnvelope.request.intent.name === 'AMAZON.HelpIntent';
},
handle(handlerInput) {
const speechText = 'If you are getting the answers as undifiend this could be due to a connection issue with GMail please reconnect and this should fix this issue';
 
        return handlerInput.responseBuilder
            .speak(speechText)
            .reprompt(speechText)
            .getResponse();
    }
 
};
const CancelAndStopIntentHandler = {
canHandle(handlerInput) {
return handlerInput.requestEnvelope.request.type === 'IntentRequest'
&& (handlerInput.requestEnvelope.request.intent.name === 'AMAZON.CancelIntent'
|| handlerInput.requestEnvelope.request.intent.name === 'AMAZON.StopIntent');
},
handle(handlerInput) {
const speechText = 'Goodbye!';
return handlerInput.responseBuilder
.speak(speechText)
.getResponse();
}
};
const SessionEndedRequestHandler = {
canHandle(handlerInput) {
return handlerInput.requestEnvelope.request.type === 'SessionEndedRequest';
},
handle(handlerInput) {
// Any cleanup logic goes here.
return handlerInput.responseBuilder.getResponse();
}
};
// The intent reflector is used for interaction model testing and debugging.
// It will simply repeat the intent the user said. You can create custom handlers
// for your intents by defining them above, then also adding them to the request
// handler chain below.
const IntentReflectorHandler = {
canHandle(handlerInput) {
return handlerInput.requestEnvelope.request.type === 'IntentRequest';
},
handle(handlerInput) {
const intentName = handlerInput.requestEnvelope.request.intent.name;
const speechText = `You just triggered ${intentName}`;
 
        return handlerInput.responseBuilder
            .speak(speechText)
            //.reprompt('add a reprompt if you want to keep the session open for the user to respond')
            .getResponse();
    }
 
};
// Generic error handling to capture any syntax or routing errors. If you receive an error
// stating the request handler chain is not found, you have not implemented a handler for
// the intent being invoked or included it in the skill builder below.
const ErrorHandler = {
canHandle() {
return true;
},
handle(handlerInput, error) {
console.log(`~~~~ Error handled: ${error.message}`);
const speechText = `Sorry, I couldn't understand what you said. Please try again.`;
 
        return handlerInput.responseBuilder
            .speak(speechText)
            .reprompt(speechText)
            .getResponse();
    }
 
};
// This handler acts as the entry point for your skill, routing all request and response
// payloads to the handlers above. Make sure any new handlers or interceptors you've
// defined are included below. The order matters - they're processed top to bottom.
exports.handler = Alexa.SkillBuilders.custom()
.addRequestHandlers(
LaunchRequestHandler,
MailCountIntentHandler,
MailTotalIntentHandler,
MailCountTimeIntentHandler,
YesIntentHandler,
NoIntentHandler,
NextIntentHandler,
PreviousIntentHandler,
MessageIntentHandler,
FromIntentHandler,
HelpIntentHandler,
CancelAndStopIntentHandler,
SessionEndedRequestHandler,
IntentReflectorHandler) // make sure IntentReflectorHandler is last so it doesn't override your custom intent handlers
.addErrorHandlers(
ErrorHandler)
.lambda();